#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""This is a Marlin example."""
from __future__ import print_function
from __future__ import absolute_import
from DIRAC.Core.Base import Script
Script.parseCommandLine()
from ILCDIRAC.Interfaces.API.NewInterface.UserJob import UserJob
from ILCDIRAC.Interfaces.API.NewInterface.Applications import Marlin
from ILCDIRAC.Interfaces.API.DiracILC import DiracILC
d = DiracILC(True, "repo.rep")

# In case one wants a loop: comment the folowing.
# for i in range(2):
j = UserJob()
j.setJobGroup("Tutorial")
j.setName("example")  # %i)

ma = Marlin()
ma.setVersion("v0111Prod")
ma.setSteeringFile("clic_ild_cdr_steering.xml")
ma.setGearFile("clic_ild_cdr.gear")
ma.setInputFile("LFN:/ilc/prod/clic/3tev/gghad/ILD/SIM/00000187/000/gghad_sim_187_97.slcio")
outdst = "toto.dst.slcio"  # % i
outrec = "toto.rec.slcio"  # % i
ma.setOutputDstFile(outdst)
ma.setOutputRecFile(outrec)

res = j.append(ma)
if not res['OK']:
  print(res['Message'])
  exit(1)

j.setOutputData([outdst, outrec], "some/path", "KEK-SRM")
j.setOutputSandbox("*.log")
j.dontPromptMe()
j.submit(d)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Run Marlin.

ILCDIRAC.Workflow.Modules.MarlinAnalysis Called by Job Agent.

:since: Feb 9, 2010

:author: Stephane Poss
:author: Przemyslaw Majewski
"""

from __future__ import absolute_import
import glob
import os
import shutil

from DIRAC.Core.Utilities.Subprocess import shellCall
from DIRAC import S_OK, S_ERROR, gLogger

from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase
from ILCDIRAC.Core.Utilities.CombinedSoftwareInstallation import getSoftwareFolder, getEnvironmentScript
from ILCDIRAC.Core.Utilities.PrepareOptionFiles import prepareXMLFile, getNewLDLibs
from ILCDIRAC.Core.Utilities.resolvePathsAndNames import resolveIFpaths, getProdFilename
from ILCDIRAC.Core.Utilities.PrepareLibs import removeLibc
from ILCDIRAC.Core.Utilities.FindSteeringFileDir import getSteeringFileDirName
from ILCDIRAC.Workflow.Utilities.DD4hepMixin import DD4hepMixin

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class MarlinAnalysis(DD4hepMixin, ModuleBase):
  """Define the Marlin analysis part of the workflow."""

  def __init__(self):
    super(MarlinAnalysis, self).__init__()
    self.enable = True
    self.STEP_NUMBER = ''
    self.result = S_ERROR()
    self.inputGEAR = ''
    self.outputREC = ''
    self.outputDST = ''
    self.applicationName = "Marlin"
    self.eventstring = ['ProgressHandler', 'event', 'EventNumber', 'StatusMonitor']
    self.envdict = {}
    self.ProcessorListToUse = []
    self.ProcessorListToExclude = []
    self.detectorModel = None

  def applicationSpecificInputs(self):
    """Resolve all input variables for the module here.

    :return: S_OK()
    """

    if 'ParametricInputSandbox' in self.workflow_commons:
      paramsb = self.workflow_commons['ParametricInputSandbox']
      if not isinstance(paramsb, list):
        if paramsb:
          paramsb = paramsb.split(";")
        else:
          paramsb = []

      self.InputFile += paramsb

    # Backward compat needed, cannot remove yet.
    self.outputREC = self.step_commons.get('outputREC', self.outputREC)
    self.outputDST = self.step_commons.get('outputDST', self.outputDST)

    if 'IS_PROD' in self.workflow_commons:
      if self.workflow_commons["IS_PROD"] and not self.OutputFile:
        if 'ProductionOutputData' in self.workflow_commons:
          outputlist = self.workflow_commons['ProductionOutputData'].split(";")
          LOG.info('Found ProductionOutputData in WF Commons', str(outputlist))
          for obj in outputlist:
            if obj.lower().count("_rec_"):
              self.outputREC = os.path.basename(obj)
            elif obj.lower().count("_dst_"):
              self.outputDST = os.path.basename(obj)
            elif obj.lower().count("_sim_"):
              self.InputFile = [os.path.basename(obj)]
        else:
          prodID = int(self.workflow_commons['PRODUCTION_ID'])
          jobID = int(self.workflow_commons["JOB_ID"])
          self.outputREC = getProdFilename(self.outputREC, prodID, jobID)
          self.outputDST = getProdFilename(self.outputDST, prodID, jobID)
          self.InputFile = [getProdFilename(self.InputFile, prodID, jobID)]
    LOG.info('OutputDST:', self.outputDST)
    LOG.info('OutputREC:', self.outputREC)
    if not self.InputFile and self.InputData:
      for files in self.InputData:
        if files.lower().find(".slcio") > -1:
          self.InputFile.append(files)

    return S_OK('Parameters resolved')

  def runIt(self):
    """Called by Agent.

    Execute the following:
      - resolve where the soft was installed
      - prepare the list of file to feed Marlin with
      - create the XML file on which Marlin has to run, done by :any:`prepareXMLFile`
      - run Marlin and catch the exit code

    :return: S_OK(), S_ERROR()
    """
    self.result = S_OK()
    if not self.platform:
      self.result = S_ERROR('No ILC platform selected')
    elif not self.applicationLog:
      self.result = S_ERROR('No Log file provided')
    if not self.result['OK']:
      LOG.error("Failed to resolve input parameters:", self.result["Message"])
      return self.result

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      return S_OK('%s should not proceed as previous step did not end properly' % self.applicationName)

    # get the path to the detector model, either local or from the software
    compactFile = None
    if self.detectorModel:
      resXML = self._getDetectorXML()
      if not resXML['OK']:
        LOG.error("Could not obtain the detector XML file: ", resXML["Message"])
        return resXML
      compactFile = resXML['Value']

    res = getEnvironmentScript(self.platform, "marlin", self.applicationVersion, self.getEnvScript)
    if not res['OK']:
      LOG.error("Failed to get the env script")
      return res
    env_script_path = res["Value"]

    res = self._getInputFiles()
    if not res['OK']:
      LOG.error("Failed getting input files:", res['Message'])
      return res
    listofslcio = res['Value']

    finalXML = "marlinxml_" + self.STEP_NUMBER + ".xml"

    steeringfiledirname = ''
    res = getSteeringFileDirName(self.platform, "marlin", self.applicationVersion)
    if res['OK']:
      steeringfiledirname = res['Value']
    else:
      LOG.warn('Could not find the steering file directory', res['Message'])

    # Handle PandoraSettings.xml
    pandorasettings = 'PandoraSettings.xml'
    if not os.path.exists(pandorasettings):
      if steeringfiledirname and os.path.exists(os.path.join(steeringfiledirname, pandorasettings)):
        try:
          shutil.copy(os.path.join(steeringfiledirname, pandorasettings),
                      os.path.join(os.getcwd(), pandorasettings))
        except EnvironmentError as x:
          LOG.warn('Could not copy PandoraSettings.xml, exception: %s' % x)

    if self.inputGEAR:
      self.inputGEAR = os.path.basename(self.inputGEAR)
      if self.inputGEAR and not os.path.exists(self.inputGEAR) and steeringfiledirname \
         and os.path.exists(os.path.join(steeringfiledirname, self.inputGEAR)):
        self.inputGEAR = os.path.join(steeringfiledirname, self.inputGEAR)

    self.SteeringFile = os.path.basename(self.SteeringFile)
    if not os.path.exists(self.SteeringFile):
      if steeringfiledirname:
        if os.path.exists(os.path.join(steeringfiledirname, self.SteeringFile)):
          self.SteeringFile = os.path.join(steeringfiledirname, self.SteeringFile)
    if not self.SteeringFile:
      LOG.error("Steering file not defined, shouldn't happen!")
      return S_ERROR("Could not find steering file")

    resOver = self._checkRunOverlay()
    if not resOver['OK']:
      return resOver
    overlayParam = resOver['Value']

    res = prepareXMLFile(finalXML, self.SteeringFile, self.inputGEAR, listofslcio,
                         self.NumberOfEvents, self.OutputFile, self.outputREC, self.outputDST,
                         self.debug,
                         dd4hepGeoFile=compactFile,
                         overlayParam=overlayParam,
                        )
    if not res['OK']:
      LOG.error('Something went wrong with XML generation because %s' % res['Message'])
      self.setApplicationStatus('Marlin: something went wrong with XML generation')
      return res

    res = self.prepareMARLIN_DLL(env_script_path)
    if not res['OK']:
      LOG.error('Failed building MARLIN_DLL:', res['Message'])
      self.setApplicationStatus('Failed to setup MARLIN_DLL')
      return S_ERROR('Something wrong with software installation')
    marlin_dll = res["Value"]

    self.result = self.runMarlin(finalXML, env_script_path, marlin_dll)
    if not self.result['OK']:
      LOG.error('Something wrong during running:', self.result['Message'])
      self.setApplicationStatus('Error during running %s' % self.applicationName)
      return S_ERROR('Failed to run %s' % self.applicationName)

    #self.result = {'OK':True,'Value':(0,'Disabled Execution','')}
    resultTuple = self.result['Value']
    if not os.path.exists(self.applicationLog):
      LOG.error("Something went terribly wrong, the log file is not present")
      self.setApplicationStatus('%s failed terribly, you are doomed!' % (self.applicationName))
      if not self.ignoreapperrors:
        return S_ERROR('%s did not produce the expected log' % (self.applicationName))

    status = resultTuple[0]
    # stdOutput = resultTuple[1]
    # stdError = resultTuple[2]
    LOG.info("Status after the application execution is:", str(status))

    return self.finalStatusReport(status)

  def prepareMARLIN_DLL(self, env_script_path):
    """Prepare the run time environment: MARLIN_DLL in particular."""
    # to fix the MARLIN_DLL, we need to get it first
    with open("temp.sh", 'w') as script:
      script.write("#!/bin/bash\n")
      lines = []
      lines.append('source %s > /dev/null' % env_script_path)
      lines.append('echo $MARLIN_DLL')
      script.write("\n".join(lines))
    os.chmod("temp.sh", 0o755)
    res = shellCall(0, "./temp.sh")
    if not res['OK']:
      LOG.error("Could not get the MARLIN_DLL env")
      return S_ERROR("Failed getting the MARLIN_DLL")
    marlindll = res["Value"][1].rstrip()
    marlindll = marlindll.rstrip(":")
    try:
      os.remove('temp.sh')
    except EnvironmentError as e:
      LOG.warn("Failed to delete the temp file", str(e))

    if not marlindll:
      return S_ERROR("Empty MARLIN_DLL env variable!")
    # user libs
    userlib = ""

    if os.path.exists("./lib/marlin_dll"):
      for library in glob.glob("./lib/marlin_dll/*.so"):
        userlib = userlib + library + ":"

    userlib = userlib.rstrip(":")

    temp = marlindll.split(':')
    temp2 = userlib.split(":")
    lib1d = {}
    libuser = {}
    for lib in temp:
      lib1d[os.path.basename(lib)] = lib
    for lib in temp2:
      libuser[os.path.basename(lib)] = lib

    for lib1, path1 in lib1d.items():
      if lib1 in libuser:
        LOG.verbose("Duplicated lib found, removing %s" % path1)
        try:
          temp.remove(path1)
        except ValueError:
          pass

    marlindll = "%s:%s" % (":".join(temp), userlib)  # Here we concatenate the default MarlinDLL with the user's stuff
    finallist = []
    items = marlindll.split(":")
    # Care for user defined list of processors, useful when someone does not want to run the full reco
    if self.ProcessorListToUse:
      for processor in self.ProcessorListToUse:
        for item in items:
          if item.count(processor):
            finallist.append(item)
    else:
      finallist = items
    items = finallist
    # Care for user defined excluded list of processors, useful when someone does not want to run the full reco
    if self.ProcessorListToExclude:
      for item in items:
        for processor in self.ProcessorListToExclude:
          if item.count(processor):
            finallist.remove(item)
    else:
      finallist = items

    # LCFIPlus links with LCFIVertex, LCFIVertex needs to go first in the MARLIN_DLL
    plusPos = 0
    lcfiPos = 0
    for position, lib in enumerate(finallist):
      if 'libLCFIPlus' in lib:
        plusPos = position
      if 'libLCFIVertex' in lib:
        lcfiPos = position
    if plusPos < lcfiPos:  # if lcfiplus is before lcfivertex
      # swap the two entries
      finallist[plusPos], finallist[lcfiPos] = finallist[lcfiPos], finallist[plusPos]

    marlindll = ":".join(finallist)
    LOG.verbose("Final MARLIN_DLL is:", marlindll)

    return S_OK(marlindll)

  def runMarlin(self, inputxml, env_script_path, marlin_dll):
    """Actual bit of code running Marlin.

    Tomato calls this function.
    """
    scriptName = '%s_%s_Run_%s.sh' % (self.applicationName, self.applicationVersion, self.STEP_NUMBER)
    if os.path.exists(scriptName):
      os.remove(scriptName)
    script = open(scriptName, 'w')
    script.write('#!/bin/bash \n')
    script.write('#####################################################################\n')
    script.write('# Dynamically generated script to run a production or analysis job. #\n')
    script.write('#####################################################################\n')
    script.write("source %s\n" % env_script_path)
    script.write("declare -x MARLIN_DLL=%s\n" % marlin_dll)
    # needed for lcgeo detector components for example
    if os.path.exists("./lib/"):
      script.write('declare -x LD_LIBRARY_PATH=./lib:$LD_LIBRARY_PATH\n')
    if os.path.exists("./lib/lddlib"):
      script.write('declare -x LD_LIBRARY_PATH=./lib/lddlib:$LD_LIBRARY_PATH\n')
    script.write('declare -x PATH=$ROOTSYS/bin:$PATH\n')
    script.write('declare -x MARLIN_DEBUG=1\n')  # Needed for recent version of marlin (from 03 april 2013)
    # We need to make sure the PandoraSettings is in the current directory
    script.write("""
if [ -e "${PANDORASETTINGS}" ]
then
   cp $PANDORASETTINGS .
fi
""")
    script.write('echo =============================\n')
    script.write('echo LD_LIBRARY_PATH is\n')
    script.write('echo $LD_LIBRARY_PATH | tr ":" "\n"\n')
    script.write('echo =============================\n')
    script.write('echo PATH is\n')
    script.write('echo $PATH | tr ":" "\n"\n')
    script.write('echo =============================\n')
    script.write('echo MARLIN_DLL is\n')
    script.write('echo $MARLIN_DLL | tr ":" "\n"\n')
    script.write('echo =============================\n')
    if self.debug:
      script.write('echo ldd of executable is\n')
      script.write('ldd `which Marlin` \n')
      script.write('echo =============================\n')
      if os.path.exists('./lib/marlin_dll'):
        script.write('ldd ./lib/marlin_dll/*.so \n')
      if os.path.exists('./lib/lddlib'):
        script.write('ldd ./lib/lddlib/*.so \n')
      script.write('echo =============================\n')
    script.write('env | sort >> localEnv.log\n')

    if os.path.exists(inputxml):
      # check
      script.write('Marlin -c %s %s\n' % (inputxml, self.extraCLIarguments))
      # real run
      script.write('Marlin %s %s\n' % (inputxml, self.extraCLIarguments))
    else:
      script.close()
      LOG.error("Steering file missing")
      return S_ERROR("SteeringFile is missing")
    script.write('declare -x appstatus=$?\n')
    script.write('exit $appstatus\n')

    script.close()
    if os.path.exists(self.applicationLog):
      os.remove(self.applicationLog)

    os.chmod(scriptName, 0o755)
    comm = 'sh -c "./%s"' % (scriptName)
    self.setApplicationStatus('%s %s step %s' % (self.applicationName, self.applicationVersion, self.STEP_NUMBER))
    self.stdError = ''
    res = shellCall(0, comm, callbackFunction=self.redirectLogOutput, bufferLimit=20971520)
    return res

  def _getInputFiles(self):
    """Resolve the input files.

    But not if in the application definition it was decided
    that it should forget about the input.
    """
    if self.ignoremissingInput:
      return S_OK("")
    res = resolveIFpaths(self.InputFile)
    if not res['OK']:
      self.setApplicationStatus('%s: missing slcio file' % self.applicationName)
      return S_ERROR('Missing slcio file!')
    runonslcio = res['Value']

    listofslcio = " ".join(runonslcio)

    return S_OK(listofslcio)

  def _checkRunOverlay(self):
    """checks if overlay should be run and which overlay parameters are there, allows running multiple overlay input steps."""
    overlayParam = []
    for parameter in sorted(self.workflow_commons):
      if parameter.startswith('OI_') and parameter.endswith('_eventType'):
        stepNumber = int(parameter.split('_')[1])
        eventsPerBackgroundFile = self.workflow_commons.get("OI_%i_eventsPerBackgroundFile" % stepNumber)
        backgroundType = self.workflow_commons.get("OI_%i_eventType" % stepNumber)
        processorName = self.workflow_commons.get("OI_%i_processorName" % stepNumber, None)
        LOG.info("Number of Events per BackgroundFile: %d " % eventsPerBackgroundFile)
        LOG.info("Background type: %s " % backgroundType)
        overlayParam.append((backgroundType, eventsPerBackgroundFile, processorName))

    return S_OK(overlayParam)

  def getEnvScript(self, sysconfig, appname, appversion):
    """Called if CVMFS is not available."""
    res = getSoftwareFolder(sysconfig, appname, appversion)
    if not res['OK']:
      self.setApplicationStatus('Marlin: Could not find neither local area not shared area install')
      return res

    myMarlinDir = res['Value']

    # Remove libc
    removeLibc(myMarlinDir + "/LDLibs")

    # Need to fetch the new LD_LIBRARY_PATH
    new_ld_lib_path = getNewLDLibs(sysconfig, "marlin", appversion)

    marlindll = ""
    if os.path.exists("%s/MARLIN_DLL" % myMarlinDir):
      for library in os.listdir("%s/MARLIN_DLL" % myMarlinDir):
        marlindll = marlindll + "%s/MARLIN_DLL/%s" % (myMarlinDir, library) + ":"
      marlindll = "%s" % (marlindll)
    else:
      LOG.error('MARLIN_DLL folder not found, cannot proceed')
      return S_ERROR('MARLIN_DLL folder not found in %s' % myMarlinDir)

    env_script_name = "MarlinEnv.sh"
    script = open(env_script_name, "w")
    script.write("#!/bin/sh\n")
    script.write('##########################################################\n')
    script.write('# Dynamically generated script to create env for Marlin. #\n')
    script.write('##########################################################\n')
    script.write("declare -x PATH=%s/Executable:$PATH\n" % myMarlinDir)
    script.write('declare -x ROOTSYS=%s/ROOT\n' % (myMarlinDir))
    script.write('declare -x LD_LIBRARY_PATH=$ROOTSYS/lib:%s/LDLibs:%s\n' % (myMarlinDir, new_ld_lib_path))
    script.write("declare -x MARLIN_DLL=%s\n" % marlindll)
    script.write("declare -x PANDORASETTINGS=%s/Settings/PandoraSettings.xml" % myMarlinDir)
    script.close()
    #os.chmod(env_script_name, 0755)
    return S_OK(os.path.abspath(env_script_name))



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""
StdhepCut: apply generator level cuts after pythia or whizard
"""
from __future__ import absolute_import
import types

from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC import S_OK, S_ERROR, gLogger

from ILCDIRAC.Interfaces.API.NewInterface.LCApplication import LCApplication
import six

LOG = gLogger.getSubLogger(__name__)
__RCSID__ = "$Id$"


class StdhepCut(LCApplication):
  """Call stdhep cut after whizard or pythia.

  Usage:

  >>> py = Pythia()
  ...
  >>> cut = StdhepCut()
  >>> cut.getInputFromApp(py)
  >>> cut.setSteeringFile("mycut.cfg")
  >>> cut.setMaxNbEvts(10)
  >>> cut.setNbEvtsPerFile(10)
  """

  def __init__(self, paramdict=None):
    self.maxNumberOfEvents = 0
    self.numberOfEventsPerFile = 0
    self.selectionEfficiency = 0
    self.inlineCuts = ""
    self.storeFile = True
    self.fileMask = '*.stdhep'
    super(StdhepCut, self).__init__(paramdict)

    self.appname = 'stdhepcut'
    self._modulename = 'StdHepCut'
    self._moduledescription = 'Module to cut on Generator (Whizard of PYTHIA)'
    self.datatype = 'gen'

  def setMaxNbEvts(self, nbevts):
    """Max number of events passing cuts to write (number of events in the final file)

    :param int nbevts: Maximum number of events passing cuts to write
    """
    self._checkArgs({'nbevts': int})
    self.maxNumberOfEvents = nbevts

  def setNbEvtsPerFile(self, nbevts):
    """Number of events per file (not used)

    :param int nbevts: Number of events to keep in each file.
    """
    self._checkArgs({'nbevts': int})
    self.numberOfEventsPerFile = nbevts

  def setSelectionEfficiency(self, efficiency):
    """Selection efficiency of your cuts, needed to determine the number of files that will be created.

    :param float efficiency: Cut efficiency
    """
    self._checkArgs({'efficiency': float})
    self.selectionEfficiency = efficiency

  def setInlineCuts(self, cutsstring):
    """Define cuts directly, not by specifying a file.

    :param str cutsstring: Cut string. Can be multiline
    """
    self._checkArgs({'cutsstring': (str,)})

    self.inlineCuts = ";".join([cut.strip() for cut in cutsstring.strip().split("\n")])

  def setFileMask(self, mask):
    """Define string to use to look for stdhep files.

    :param str mask: mask to find input files for stdhepcut, default ``*.stdhep'``
    """
    self._checkArgs({'mask': (str,)})
    self.fileMask = mask

  def setStoreFile(self, storeFile):
    """Enable or disable setting of output data.

    :param bool storeFile: Set to False if StdhepCut is not the last step in production job.
    """
    self._checkArgs({'storeFile': bool})
    self.storeFile = storeFile

  def _applicationModule(self):
    m1 = self._createModuleDefinition()
    m1.addParameter(Parameter("MaxNbEvts", 0, "int", "", "", False, False, "Number of events to read"))
    m1.addParameter(Parameter("debug", False, "bool", "", "", False, False, "debug mode"))
    m1.addParameter(Parameter("inlineCuts", "", "string", "", "", False, False, "Inline cuts"))
    m1.addParameter(Parameter('fileMask', '', 'string', '', '', False, False, 'File Mask'))

    return m1

  def _applicationModuleValues(self, moduleinstance):
    moduleinstance.setValue("MaxNbEvts", self.maxNumberOfEvents)
    moduleinstance.setValue("debug", self.debug)
    moduleinstance.setValue("inlineCuts", self.inlineCuts)
    moduleinstance.setValue('fileMask', self.fileMask)

  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setUserJobFinalization(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('userjobmodules failed')
    return S_OK()

  def _prodjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setOutputComputeDataList(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('prodjobmodules failed')
    return S_OK()

  def _checkConsistency(self, job=None):
    if not self.steeringFile and not self.inlineCuts:
      return S_ERROR("Cuts not specified")
    if self.steeringFile and self.inlineCuts:
      LOG.notice("You specifed a cuts file and InlineCuts. InlineCuts has precedence.")
    # elif not self.SteeringFile.lower().count("lfn:") and not os.path.exists(self.SteeringFile):
    # res = Exists(self.SteeringFile)
    # if not res['OK']:
    #   return res

    if not self.maxNumberOfEvents:
      return S_ERROR("You did not specify how many events you need to keep per file (MaxNbEvts)")

    if not self.selectionEfficiency:
      return S_ERROR('You need to know the selection efficiency of your cuts')

    if self._jobtype != 'User' and self.storeFile:
      self._listofoutput.append({"outputFile": "@{OutputFile}", "outputPath": "@{OutputPath}",
                                 "outputDataSE": '@{OutputSE}'})
      self.prodparameters['nbevts_kept'] = self.maxNumberOfEvents
      self.prodparameters['cut_file'] = self.steeringFile

    # res = self._checkRequiredApp() ##Check that job order is correct
    # if not res['OK']:
    #  return res

    return S_OK()

  def _checkFinalConsistency(self):
    """Final check of consistency: check that there are enough events generated."""
    if not self.numberOfEvents:
      return S_ERROR('Please specify the number of events that will be generated in that step')

    kept = self.numberOfEvents * self.selectionEfficiency
    if kept < 2 * self.maxNumberOfEvents:
      return S_ERROR("You don't generate enough events")

    return S_OK()

  def _checkWorkflowConsistency(self):
    return self._checkRequiredApp()

  def _resolveLinkedStepParameters(self, stepinstance):
    if isinstance(self._linkedidx, six.integer_types):
      self._inputappstep = self._jobsteps[self._linkedidx]
    if self._inputappstep:
      stepinstance.setLink("InputFile", self._inputappstep.getType(), "OutputFile")
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Unit tests for the CalibrationService."""

from __future__ import print_function
from __future__ import absolute_import
import unittest
import pytest
import os
import shutil
import time
from datetime import datetime
from datetime import timedelta
from xml.etree import ElementTree as et
from shutil import copyfile
from DIRAC import S_OK, S_ERROR
from mock import patch
from mock import MagicMock as Mock
from DIRAC.Core.DISET.RequestHandler import RequestHandler
from ILCDIRAC.CalibrationSystem.Service.CalibrationHandler import CalibrationHandler
from ILCDIRAC.CalibrationSystem.Service.CalibrationRun import CalibrationRun
from Tests.Utilities.GeneralUtils import \
    assertEqualsImproved, assertDiracFailsWith, assertDiracSucceeds, \
    assertDiracSucceedsWith_equals
from ILCDIRAC.CalibrationSystem.Service.DetectorSettings import createCalibrationSettings
from ILCDIRAC.CalibrationSystem.Utilities.functions import readParametersFromSteeringFile
from six.moves import filter
from six.moves import range

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.CalibrationSystem.Service.CalibrationHandler'


@pytest.fixture
def readParameterDict():
  """Read parameter list."""
  from ILCDIRAC.CalibrationSystem.Utilities.functions import readParameterDict

  fileDir = os.path.join(os.environ['ILCDIRAC_BASE_FOLDER'], 'src', 'ILCDIRAC',
                         'CalibrationSystem', 'Utilities', 'auxiliaryFiles')
  fileToRead = os.path.join(fileDir, 'parameterListMarlinSteeringFile.txt')
  parDict = readParameterDict(fileToRead)
  for iKey in parDict.keys():
    if 'CalibrECAL' in iKey:
      parDict[iKey] = '1.0 1.0'
    else:
      parDict[iKey] = 1.0
  return parDict


@pytest.fixture
def calibHandler():
  """Create calibration handler."""
  CalibrationHandler.initializeHandler(None)
  RequestHandler._rh__initializeClass(Mock(), Mock(), Mock(), Mock())
  calibHandler = CalibrationHandler({}, Mock())
  calibHandler.initialize()
  print('Init CalibrationHandler. List of active calibrations: %s' % list(CalibrationHandler.activeCalibrations.keys()))
  yield calibHandler

  if os.path.exists('status'):
    try:
      os.remove('status')
    except OSError:
      print("Failed to clean up CalibrationHandler fixture. Cannot delete file 'status'")
      assert False
  # clean up output directory
  for iCalID in list(CalibrationHandler.activeCalibrations.keys()):
    try:
      dirToDelete = 'calib%s' % iCalID
      if os.path.exists(dirToDelete):
        shutil.rmtree(dirToDelete)
        #  pass
    except EnvironmentError as e:
      print("Failed to delete directory: %s" % dirToDelete, str(e))
      assert False
  CalibrationHandler.activeCalibrations = {}
  CalibrationHandler.calibrationCounter = 0


@pytest.fixture
def copiedFccSteeringFile():
  """Copy steering file for testing."""
  calibID = 1
  workdirName = 'calib%s' % calibID
  if not os.path.exists(workdirName):
    os.makedirs(workdirName)
  src = os.path.join(os.environ['ILCDIRAC_BASE_FOLDER'], 'Tests', 'Files', 'fccReconstruction.xml')
  copyfile(src, '%s/fccReconstruction.xml' % workdirName)
  yield workdirName
  try:
    if os.path.exists(workdirName):
      shutil.rmtree(workdirName)
    #  pass
  except EnvironmentError as e:
    print("Failed to delete directory: %s; ErrMsg: %s" % (workdirName, str(e)))
    assert False


def test_returnTypesOfExportFunctionsOfHandler(mocker, calibHandler):
  """Test returns of export_<> functions.

  Test that returns of all export_<> functions of the handler, which doesn't have input arguments, are standard python
  structures.
  """
  mocker.patch.object(calibHandler, '_getUsernameAndGroup', new=Mock(
      return_value={'OK': True, 'Value': {'username': 'correctUserName', 'group': 'correctUserGroup'}}))
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.convert_and_execute',
               side_effect=mimic_convert_and_execute)
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.saveCalibrationRun', new=Mock(return_value=S_OK()))
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationHandler.saveCalibrationRun',
               new=Mock(return_value=S_OK()))

  calibSetting = createCalibrationSettings('CLD')
  calibSetting.settingsDict['numberOfJobs'] = 200

  for i in range(1, 4):
    calibRun = CalibrationRun(i, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
    mocker.patch.object(calibRun, 'ops', new=Mock(return_value=Mock(), name='Class'))
    mocker.patch.object(calibRun, 'endCurrentStep', new=Mock(return_value=S_OK()))
    calibRun.proxyUserName = 'correctUserName'
    calibRun.proxyUserGroup = 'correctUserGroup'
    calibRun.calibrationFinished = False
    CalibrationHandler.activeCalibrations[i] = calibRun

  allInstanceMembers = dir(calibHandler)
  import re
  myPattern = re.compile('export_*')
  exportFunctionMembers = list(filter(myPattern.match, allInstanceMembers))
  calibHandler.export_killCalibration(1, 'killed by user')

  try:
    # Python 3
    from inspect import getfullargspec as sig
  except ImportError:
    # Python 2
    # example of return: ArgSpec(args=['self'], varargs=None, keywords=None, defaults=None)
    from inspect import getargspec as sig

  for iFunc in exportFunctionMembers:
    if iFunc.endswith(('whoami', 'ping')):
      continue
    methodToCall = getattr(calibHandler, iFunc)
    res = sig(methodToCall)
    TestThisFunction = False
    if len(res.args) == 1:
      if res.args[0] == "self":
        TestThisFunction = True
    elif res.defaults is not None:
      if len(res.args) == len(res.defaults) + 1:
        TestThisFunction = True
    if not TestThisFunction:
      continue
    #  print("Test function: %s" % iFunc)
    res = methodToCall()
    assert isinstance(res, dict)
    assert 'OK' in res
    if res['OK']:
      assert 'Value' in res
      assert isinstance(res['Value'], (int, float, str, dict, list, tuple)) or res['Value'] is None
      if isinstance(res['Value'], (list, tuple)):
        print(res['Value'])
        assert isinstance(res['Value'][0], (int, float, str, dict))
    else:
      assert 'Message' in res
      assert not res['Message']


def test_createCalibration(calibHandler):
  """Test createCalibration."""
  cldSettings = createCalibrationSettings('CLD')
  clicSettings = createCalibrationSettings('CLIC')
  inputData = {'zuds': [], 'gamma': [], 'muon': [], 'kaon': []}
  numberOfEventsPerFile = {'zuds': 33, 'gamma': 33, 'muon': 33, 'kaon': 33}

  # wrong input: missing argument in the first input dict
  res = calibHandler.export_createCalibration(inputData, numberOfEventsPerFile, clicSettings.settingsDict)
  assert not res['OK']
  assert "Following settings have None values: ['outputSE']. All settings have to be set up." in res['Message']
  clicSettings.settingsDict['outputSE'] = 'dummy_outputSE'
  res = calibHandler.export_createCalibration(inputData, numberOfEventsPerFile, clicSettings.settingsDict)
  assert not res['OK']
  assert "Following settings have None values: ['outputPath']. All settings have to be set up." in res['Message']
  cldSettings.settingsDict['outputPath'] = 'dummy_outputPath'
  clicSettings.settingsDict['outputPath'] = 'dummy_outputPath'
  cldSettings.settingsDict['outputSE'] = 'dummy_outputSE'

  # wrong input: missing argument in the first input dict
  res = calibHandler.export_createCalibration(
      {'zuds': [], 'gamma': [], 'muon': []}, numberOfEventsPerFile, clicSettings.settingsDict)
  assert not res['OK']
  assert "doesn't contains all required fields." in res['Message']

  # wrong input: unused extra argument in the first input dict
  res = calibHandler.export_createCalibration({'zuds': [], 'gamma': [], 'muon': [], 'kaon': [], 'wonderParticle': []},
                                              numberOfEventsPerFile, clicSettings.settingsDict)
  assert not res['OK']
  assert "doesn't contains all required fields." in res['Message']

  # wrong input: unused extra argument in the third input dict
  wrongSettings = createCalibrationSettings('CLD').settingsDict
  wrongSettings['outputPath'] = 'dummy_outputPath'
  wrongSettings['outputSE'] = 'dummy_outputSE'
  wrongSettings['dummy'] = 'dummy'
  res = calibHandler.export_createCalibration(inputData, numberOfEventsPerFile, wrongSettings)
  assert not res['OK']
  assert "doesn't contains all required fields." in res['Message']

  # wrong input: missing argument in the third input dict
  wrongSettings = createCalibrationSettings('CLD').settingsDict
  wrongSettings['outputPath'] = 'dummy_outputPath'
  wrongSettings['outputSE'] = 'dummy_outputSE'
  wrongSettings.pop('detectorModel', None)
  res = calibHandler.export_createCalibration(inputData, numberOfEventsPerFile, wrongSettings)
  assert not res['OK']
  assert "doesn't contains all required fields." in res['Message']

  # wrong input: swapped input dictionaries
  res = calibHandler.export_createCalibration(cldSettings.settingsDict, inputData, numberOfEventsPerFile)
  assert not res['OK']
  assert "doesn't contains all required fields." in res['Message']

  # wrong input: wrong values in numberOfEventsPerFile
  wrongNumberOfEventsPerFile = {'zuds': 0, 'gamma': 33, 'muon': 33, 'kaon': 33}
  res = calibHandler.export_createCalibration(inputData, wrongNumberOfEventsPerFile, clicSettings.settingsDict)
  assert not res['OK']
  assert "numberOfEventsPerFile" in res['Message']

  # wrong input: wrong values in numberOfEventsPerFile
  wrongNumberOfEventsPerFile = {'zuds': 1, 'gamma': 33.5, 'muon': 33, 'kaon': 33}
  res = calibHandler.export_createCalibration(inputData, wrongNumberOfEventsPerFile, clicSettings.settingsDict)
  assert not res['OK']
  assert "numberOfEventsPerFile" in res['Message']

  # nJobs > nEvents (0 files for kaons)
  inputData = {'zuds': ["dummy.slcio"], 'gamma': ["dummy.slcio"], 'muon': ["dummy.slcio"], 'kaon': []}
  numberOfEventsPerFile = {'zuds': 133, 'gamma': 133, 'muon': 133, 'kaon': 133}
  res = calibHandler.export_createCalibration(inputData, numberOfEventsPerFile, clicSettings.settingsDict)
  assert not res['OK']
  assert 'kaon' in res['Message']

  # nJobs > nEvents (33 events for muons while nJobs = 100)
  inputData = {'zuds': ["dummy.slcio"], 'gamma': ["dummy.slcio"], 'muon': ["dummy.slcio"], 'kaon': ["dummy.slcio"]}
  numberOfEventsPerFile = {'zuds': 133, 'gamma': 133, 'muon': 33, 'kaon': 133}
  res = calibHandler.export_createCalibration(inputData, numberOfEventsPerFile, clicSettings.settingsDict)
  assert not res['OK']
  assert 'muon' in res['Message']


def addPfoAnalysisProcessor(mainSteeringMarlinRecoFile):
  """Add pfoAnalysis processor to the steering file."""
  mainTree = et.ElementTree()
  mainTree.parse(mainSteeringMarlinRecoFile)
  mainRoot = mainTree.getroot()

  # FIXME TODO properly find path to the file
  # this file should only contains PfoAnalysis processor
  import ILCDIRAC.CalibrationSystem.Utilities as utilities
  pfoAnalysisProcessorFile = os.path.join(utilities.__path__[0], 'auxiliaryFiles/pfoAnalysis.xml')
  if not os.path.exists(pfoAnalysisProcessorFile):
    return S_ERROR("cannot find xml file with pfoAnalysis processor")
  tmpTree = et.parse(pfoAnalysisProcessorFile)
  elementToAdd = tmpTree.getroot()

  if 'MyPfoAnalysis' not in (iEl.attrib['name'] for iEl in mainRoot.iter('processor')):
    tmp1 = mainRoot.find('execute')
    c = et.Element("processor name=\"MyPfoAnalysis\"")
    tmp1.append(c)
    mainRoot.append(elementToAdd)
    #  mainTree.write(mainSteeringMarlinRecoFile)
    root = mainTree.getroot()
    root_str = et.tostring(root)
    # TODO FIXME why write to "test_<fileName>" file???
    #  with open('test_' + mainSteeringMarlinRecoFile, "w") as of:
    with open(mainSteeringMarlinRecoFile, "w") as of:
      of.write(root_str.decode())
  return S_OK()


def test_readInitialParameterDict(copiedFccSteeringFile, mocker):
  """Test readInitialParameterDict."""
  calibSetting = createCalibrationSettings('CLD')
  calibSetting.settingsDict['DDPandoraPFANewProcessorName'] = 'MyDDMarlinPandora_10ns'
  calibSetting.settingsDict['DDCaloDigiName'] = 'MyDDCaloDigi_10ns'
  tmpMock = Mock(name='instance')
  tmpMock.getFile.return_value = S_OK()
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.DataManager',
               new=Mock(return_value=tmpMock, name='Class'))
  calibID = int(copiedFccSteeringFile.split('calib')[-1])
  newRun = CalibrationRun(calibID, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  res = newRun.readInitialParameterDict()
  assert res['OK']
  # check if MaxClusterEnergyToApplySoftComp string have been added to the steering file
  tmpKey = (".//processor[@name='%s']/parameter[@name='MaxClusterEnergyToApplySoftComp']"
            % calibSetting.settingsDict['DDPandoraPFANewProcessorName'])
  tmpDict = {tmpKey: None}
  res = readParametersFromSteeringFile('calib' + str(calibID) + '/fccReconstruction.xml', tmpDict)
  assert res['OK']
  assert tmpDict == {tmpKey: '0'}
  # check case when nEcalThickLayers are not zero
  calibSetting.settingsDict['nEcalThinLayers'] = 40
  calibSetting.settingsDict['nEcalThickLayers'] = 34
  calibSetting.settingsDict['ecalResponseCorrectionForThickLayers'] = 2.4
  newRun = CalibrationRun(calibID, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  res = newRun.readInitialParameterDict()
  assert res['OK']
  tmpKey = ".//processor[@name='%s']/parameter[@name='ECALLayers']" % calibSetting.settingsDict['DDCaloDigiName']
  tmpDict = {tmpKey: None}
  res = readParametersFromSteeringFile('calib' + str(calibID) + '/fccReconstruction.xml', tmpDict)
  assert res['OK']
  assert tmpDict[tmpKey] == '%s %s' % (calibSetting.settingsDict['nEcalThinLayers'],
                                       calibSetting.settingsDict['nEcalThinLayers']
                                       + calibSetting.settingsDict['nEcalThickLayers'] + 1)


def test_initializeHandler(mocker):
  """Test initializeHandler."""
  mocker.patch('%s.glob.glob' % MODULE_NAME, new=Mock(return_value=['calib2', 'calib4', 'calib78']))
  mocker.patch('%s.loadCalibrationRun' % MODULE_NAME, new=Mock())
  mocker.patch.object(CalibrationHandler, 'loadStatus', new=Mock(return_value=0))

  res = CalibrationHandler.initializeHandler(None)
  assert not res['OK']

  mocker.patch.object(CalibrationHandler, 'loadStatus', new=Mock(return_value=78))
  res = CalibrationHandler.initializeHandler(None)
  assert res['OK']


def mimic_convert_and_execute(inList, _=''):
  """Mimic expected output."""
  from ILCDIRAC.CalibrationSystem.Service.CalibrationRun import CalibrationPhase
  if True in ['ECal_Digi_Extract.py' in str(iEl) for iEl in inList] and True in ['Mean' in str(iEl) for iEl in inList]:
    return S_OK((0, '%s\n' % CalibrationPhase.sampleEnergyFromPhase(CalibrationPhase.ECalDigi)))
  elif (True in ['HCal_Digi_Extract.py' in str(iEl) for iEl in inList]
        and True in ['Mean' in str(iEl) for iEl in inList]):
    return S_OK((0, '%s\n' % CalibrationPhase.sampleEnergyFromPhase(CalibrationPhase.HCalDigi)))
  elif True in ['EM_Extract.py' in str(iEl) for iEl in inList] and True in ['Mean' in str(iEl) for iEl in inList]:
    return S_OK((0, '%s\n' % CalibrationPhase.sampleEnergyFromPhase(CalibrationPhase.ElectroMagEnergy)))
  elif True in ['Had_Extract.py' in str(iEl) for iEl in inList] and True in ['FOM' in str(iEl) for iEl in inList]:
    return S_OK((0, '%s\n' % CalibrationPhase.sampleEnergyFromPhase(CalibrationPhase.HadronicEnergy)))
  elif _ != '':
    return 6.66
  else:
    return S_OK((0, '6.66\n'))


def test_endCurrentStepBasicWorkflow(copiedFccSteeringFile, readParameterDict, mocker):
  """Test endCurrentStepBasicWorkflow."""
  opsMock = Mock(name='instance')
  opsMock.getValue.return_value = 'dummy'
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.Operations',
               new=Mock(return_value=opsMock, name='Class'))
  #  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.updateSteeringFile', new=Mock(return_value=S_OK()))
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.convert_and_execute',
               side_effect=mimic_convert_and_execute)
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.saveCalibrationRun', new=Mock(return_value=S_OK()))

  calibSetting = createCalibrationSettings('CLD')
  calibSetting.settingsDict['DDPandoraPFANewProcessorName'] = 'MyDDMarlinPandora_10ns'
  calibSetting.settingsDict['DDCaloDigiName'] = 'MyDDCaloDigi_10ns'
  tmpMock = Mock(name='instance')
  tmpMock.getFile.return_value = S_OK()
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.DataManager',
               new=Mock(return_value=tmpMock, name='Class'))
  steeringFileName = copiedFccSteeringFile
  calibID = int(steeringFileName.split('calib')[-1])
  addPfoAnalysisProcessor('%s/%s' % (steeringFileName, calibSetting.settingsDict['steeringFile']))
  calibSetting.settingsDict['platform'] = "x86_64-slc5-gcc43-opt"
  newRun = CalibrationRun(calibID, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  res = newRun.readInitialParameterDict()
  assert res['OK']
  newRun.calibrationConstantsDict[".//processor[@type='PfoAnalysis']/parameter[@name='RootFile']"] = 'pfoAnalysis.root'

  mocker.patch.object(CalibrationRun, '_CalibrationRun__mergePandoraLikelihoodXmlFiles',
                      new=Mock(return_value={'OK': True}))
  stageIDSequence = []
  phaseIDSequence = []
  stepIDSequence = []
  calibFinishedSequence = []
  for _ in range(0, 13):
    stepOutcome = 'stage: %s, phase: %s, step: %s,\tcalibFinished: %s' % (
        newRun.currentStage, newRun.currentPhase, newRun.currentStep, newRun.calibrationFinished)
    print(stepOutcome)
    stageIDSequence.append(newRun.currentStage)
    phaseIDSequence.append(newRun.currentPhase)
    stepIDSequence.append(newRun.currentStep)
    calibFinishedSequence.append(newRun.calibrationFinished)
    newRun.endCurrentStep()
  assert stageIDSequence == [1, 1, 1, 1, 1, 2, 3, 3, 3, 3, 3, 3, 3]
  assert phaseIDSequence == [0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 4, 4]
  assert stepIDSequence == [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 11]
  assert calibFinishedSequence == [False, False, False, False,
                                   False, False, False, False, False, False, False, True, True]


def test_export_checkForStepIncrement(calibHandler, mocker):
  """Test export_checkForStepIncrement."""
  calibSetting = createCalibrationSettings('CLIC')
  calibRun = CalibrationRun(27, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  calibRun.calibrationFinished = True
  calibRun.calibrationEndTime = (datetime.now() - timedelta(minutes=9, seconds=59,
                                                            milliseconds=10)).strftime(calibRun.timeStampPattern)
  CalibrationHandler.activeCalibrations[27] = calibRun
  CalibrationHandler.timeToKeepCalibrationResultsInMinutes = 100

  mocker.patch.object(calibRun, 'copyResults', new=Mock(return_value=S_OK()))
  mocker.patch('%s.saveCalibrationRun' % MODULE_NAME, new=Mock(return_value=S_OK()))
  mocker.patch('%s.shutil' % MODULE_NAME, new=Mock())

  assert len(CalibrationHandler.activeCalibrations) == 1
  res = calibHandler.export_checkForStepIncrement()
  assert res['OK']
  assert len(CalibrationHandler.activeCalibrations) == 1
  CalibrationHandler.timeToKeepCalibrationResultsInMinutes = -10
  res = calibHandler.export_checkForStepIncrement()
  assert res['OK']
  assert len(CalibrationHandler.activeCalibrations) == 0


def test_finalInterimResultReceived(calibHandler, mocker):
  """Test finalInterimResultReceived."""
  calibSetting = createCalibrationSettings('CLIC')
  calibSetting.settingsDict['numberOfJobs'] = 200
  calibRun = CalibrationRun(1, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  nJobsTmp = int(calibSetting.settingsDict['numberOfJobs']
                 * calibSetting.settingsDict['fractionOfFinishedJobsNeededToStartNextStep'] - 1)
  for i in range(0, nJobsTmp):
    calibRun.stepResults[calibRun.currentStep].addResult(i, 'dummy')
  assert not calibHandler.finalInterimResultReceived(calibRun, calibRun.currentStep)
  calibRun.stepResults[calibRun.currentStep].addResult(nJobsTmp, 'dummy')
  assert calibHandler.finalInterimResultReceived(calibRun, calibRun.currentStep)

#  def test_regroupInputFile(calibHandler, mocker):
  #  inputFileDir = {'muon': ['muon1', 'muon2', 'muon3', 'muon4', 'muon5'],
  #                  'kaon': ['kaon1', 'kaon2', 'kaon3', 'kaon4', 'kaon5'],
  #                  'gamma': ['gamma1', 'gamma2', 'gamma3', 'gamma4', 'gamma5'],
  #                  'zuds': ['zuds1', 'zuds2', 'zuds3', 'zuds4', 'zuds5']}
#
#    numberOfJobs = 4
#    res = calibHandler._CalibrationHandler__regroupInputFile(inputFileDir, numberOfJobs)
#    assert res['OK']
#    groupedDict = res['Value']
#    for iKey in inputFileDir.keys():
#      assert len(groupedDict[0][iKey]) == 2
#      assert len(groupedDict[1][iKey]) == 1
#
#    numberOfJobs = 2
#    res = calibHandler._CalibrationHandler__regroupInputFile(inputFileDir, numberOfJobs)
#    assert res['OK']
#    groupedDict = res['Value']
#    for iKey in inputFileDir.keys():
#      assert len(groupedDict[0][iKey]) == 3
#      assert len(groupedDict[1][iKey]) == 2


def test_export_submitResult(calibHandler, mocker):
  """Test export_submitResult."""
  mocker.patch.object(CalibrationRun, 'submitJobs', new=Mock())
  #  mocker.patch.object(calibHandler, '_CalibrationHandler__regroupInputFile',
  #                      new=Mock(return_value={'OK': True, 'Value': []}))
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationHandler.splitFilesAcrossJobs',
               new=Mock(return_value={'OK': True, 'Value': []}))
  mocker.patch.object(calibHandler, '_getUsernameAndGroup', new=Mock(
      return_value={'OK': True, 'Value': {'username': 'oviazlo', 'group': 'ilc_users'}}))

  inputData = {'zuds': ["dummy.slcio"], 'gamma': ["dummy.slcio"], 'muon': ["dummy.slcio"], 'kaon': ["dummy.slcio"]}
  numberOfEventsPerFile = {'zuds': 133, 'gamma': 133, 'muon': 133, 'kaon': 133}

  calibSettings = createCalibrationSettings('CLIC')
  calibSettings.settingsDict['outputPath'] = 'dummy_outputPath'
  calibSettings.settingsDict['outputSE'] = 'dummy_outputSE'
  res = calibHandler.export_createCalibration(inputData, numberOfEventsPerFile, calibSettings.settingsDict)
  if not res['OK']:
    print('Error message:\t%s' % res['Message'])
    assert False

  print('CalibrationHandler.activeCalibrations.keys(): %s' % list(CalibrationHandler.activeCalibrations.keys()))

  import ILCDIRAC.CalibrationSystem.Utilities as utilities
  fileDir = utilities.__path__[0]
  fileToRead = os.path.join(fileDir, 'auxiliaryFiles/pfoAnalysis.xml')
  from ILCDIRAC.CalibrationSystem.Utilities.fileutils import binaryFileToString
  tmpFile = binaryFileToString(fileToRead)

  calibID = 1
  stageID = 2
  phaseID = 0
  stepID = 0
  workerID = 8234

  res = calibHandler.export_submitResult(calibID, stageID, phaseID, stepID, workerID, tmpFile)
  if not res['OK']:
    print(res)
    assert False
  assert res['OK']

  outFile = CalibrationHandler.activeCalibrations[calibID].stepResults[stepID].results[workerID]
  assert os.path.exists(outFile)
  print(outFile)

  import filecmp
  assert filecmp.cmp(fileToRead, outFile)


def test_mergePandoraLikelihoodXmlFiles(calibHandler, mocker):
  """Test mergePandoraLikelihoodXmlFiles."""
  import ILCDIRAC.CalibrationSystem.Utilities as utilities
  fileDir = utilities.__path__[0]

  mocker.patch.object(CalibrationRun, 'submitJobs', new=Mock())
  opsMock = Mock(name='instance')
  opsMock.getValue.return_value = os.path.join(fileDir, 'auxiliaryFiles')
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.Operations',
               new=Mock(return_value=opsMock, name='Class'))
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationHandler.splitFilesAcrossJobs',
               new=Mock(return_value={'OK': True, 'Value': []}))
  #  mocker.patch.object(calibHandler, '_CalibrationHandler__regroupInputFile',
  #                      new=Mock(return_value={'OK': True, 'Value': []}))
  mocker.patch.object(calibHandler, '_getUsernameAndGroup', new=Mock(
      return_value={'OK': True, 'Value': {'username': 'oviazlo', 'group': 'ilc_users'}}))

  calibSettings = createCalibrationSettings('CLIC')
  calibSettings.settingsDict['outputPath'] = 'dummy_outputPath'
  calibSettings.settingsDict['outputSE'] = 'dummy_outputSE'
  print('calibSettings.settingsDict: %s' % calibSettings.settingsDict)

  inputData = {'zuds': ["dummy.slcio"], 'gamma': ["dummy.slcio"], 'muon': ["dummy.slcio"], 'kaon': ["dummy.slcio"]}
  numberOfEventsPerFile = {'zuds': 133, 'gamma': 133, 'muon': 133, 'kaon': 133}
  res = calibHandler.export_createCalibration(inputData, numberOfEventsPerFile, calibSettings.settingsDict)
  print('calibHandler.export_createCalibration: %s' % res)
  if not res['OK']:
    print(res['Message'])
    assert False

  fileToRead = os.path.join(os.environ['ILCDIRAC_BASE_FOLDER'], 'Tests/Files/PandoraLikelihoodData9EBin.xml')
  if not os.path.exists(fileToRead):
    assert False
  from ILCDIRAC.CalibrationSystem.Utilities.fileutils import binaryFileToString
  tmpFile = binaryFileToString(fileToRead)

  #  calibID = 1
  #  if os.path.exists("status"):
  #    with open(fileName, 'r') as f:
  #      calibID = int(f.readlines()[0])

  calibID = 1
  stageID = 2
  phaseID = 0
  stepID = 0
  workerID = 654

  print('test_mergePandoraLikelihoodXmlFiles: List of active calibrations: %s' %
        list(CalibrationHandler.activeCalibrations.keys()))

  CalibrationHandler.activeCalibrations[calibID].currentStage = stageID
  CalibrationHandler.activeCalibrations[calibID].currentPhase = phaseID
  CalibrationHandler.activeCalibrations[calibID].currentStep = stepID

  res = calibHandler.export_submitResult(calibID, stageID, phaseID, stepID, workerID, tmpFile)
  if not res['OK']:
    print(res)
    assert False

  nFilesToMerge = 3
  for _ in range(0, nFilesToMerge - 1):
    workerID += 1
    res = calibHandler.export_submitResult(calibID, stageID, phaseID, stepID, workerID, tmpFile)
    if not res['OK']:
      print(res)
      assert False

  res = CalibrationHandler.activeCalibrations[calibID]._CalibrationRun__mergePandoraLikelihoodXmlFiles()
  if not res['OK']:
    print(res)
    assert False

  mergedFile = 'calib%s/newPandoraLikelihoodData.xml' % calibID
  assert os.path.exists(mergedFile)

  from ILCDIRAC.CalibrationSystem.Utilities.functions import searchFilesWithPattern
  inFileList = searchFilesWithPattern('calib%s/stage%s' % (calibID, stageID), '*.xml')

  import re
  diffLines = None
  with open(mergedFile) as file1:
    with open(inFileList[0]) as file2:
      diffLines = set(file1).symmetric_difference(file2)
  diffLines = list(diffLines)
  diffLines = [re.split(r'\>|\<', iLine) for iLine in diffLines]

  # since we merge a few copies of the same file, likelihood functions has to be identical in input and output files
  # the only difference has to be in the number of events in NSignalEvents and NBackgroundEvents fields (lines)
  # this is why there should be 2 unique lines in input and output files --> 4 lines in total
  assert len(diffLines) == 4

  # sum numbers from NSignalEvents and NBackgroundEvents nodes for input and output files
  nSignalEvents = []
  nBackgroundEvents = []
  for iList in diffLines:
    if iList[1] == 'NSignalEvents':
      nSignalEvents.append(sum([int(iEl) for iEl in iList[2].split()]))
    elif iList[1] == 'NBackgroundEvents':
      nBackgroundEvents.append(sum([int(iEl) for iEl in iList[2].split()]))
    else:
      pass

  # NSignalEvents_output = nFilesToMerge * NSignalEvents_input
  # NBackgroundEvents_output = nFilesToMerge * NBackgroundEvents_input
  assert ((nSignalEvents[0] == nFilesToMerge * nSignalEvents[1])
          or (nSignalEvents[1] == nFilesToMerge * nSignalEvents[0]))
  assert ((nBackgroundEvents[0] == nFilesToMerge * nBackgroundEvents[1])
          or (nBackgroundEvents[1] == nFilesToMerge * nBackgroundEvents[0]))

  #  print('nSignalEvents: %s' % nSignalEvents)
  #  print('nBackgroundEvents: %s' % nBackgroundEvents)


def test_simple_killCalibration(calibHandler, mocker):
  """Test killCalibration."""
  CalibrationHandler.activeCalibrations[27] = 'dummy27'
  CalibrationHandler.activeCalibrations[31] = 'dummy31'
  CalibrationHandler.activeCalibrations[20] = 'dummy20'

  #  mocker.patch.object(calibHandler, '_checkClientRequest', new=Mock(
  #      return_value={'OK': True, 'Value': {'username': 'correctUserName', 'group': 'correctUserGroup'}}))

  # wrong calibrationID
  res = calibHandler.export_killCalibration(2, 'dummy')
  assert res['OK']
  assert res['Value'] == 'No calibration with ID: %s was found. Active calibrations: %s' % (2, [20, 27, 31])

  # test function which accept list
  # wrong calibrationID
  res = calibHandler.export_killCalibrations([2, 3])
  assert res['OK']
  assert res['Value'] == {2: S_OK('No calibration with ID: %s was found. Active calibrations: %s' % (2, [20, 27, 31])),
                          3: S_OK('No calibration with ID: %s was found. Active calibrations: %s' % (3, [20, 27, 31]))}
  # wrong input type in the list
  res = calibHandler.export_killCalibrations([2, 'blah'])
  assert not res['OK']
  # clean up
  CalibrationHandler.activeCalibrations = {}


def test_killCalibration(calibHandler, copiedFccSteeringFile, mocker):
  """Test killCalibration."""
  from ILCDIRAC.CalibrationSystem.Utilities.functions import saveCalibrationRun
  calibID = int(copiedFccSteeringFile.split('calib')[-1])
  print('calibID: %s' % calibID)

  #  mocker.patch.object(calibHandler, '_checkClientRequest', new=Mock(
  #      return_value={'OK': True, 'Value': {'username': 'correctUserName', 'group': 'correctUserGroup'}}))
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.Operations',
               new=Mock(return_value=Mock(), name='Class'))
  mocker.patch.object(calibHandler, '_getUsernameAndGroup', new=Mock(
      return_value={'OK': True, 'Value': {'username': 'correctUserName', 'group': 'correctUserGroup'}}))
  calibSetting = createCalibrationSettings('CLIC')
  calibSetting.settingsDict['outputPath'] = 'dummy_outputPath'
  calibSetting.settingsDict['outputSE'] = 'dummy_outputSE'
  calibRun = CalibrationRun(1, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  calibRun.proxyUserName = 'wrongUserName'
  calibRun.proxyUserGroup = 'correctUserGroup'
  CalibrationHandler.activeCalibrations[1] = calibRun
  saveCalibrationRun(calibRun)

  # wrong proxyUserName
  res = calibHandler.export_killCalibration(1, 'dummy')
  assert not res['OK']
  assert res['Message'] == 'Permission denied. Calibration with ID 1 has been created by other user.'
  # wrong proxyUserName
  res = calibHandler.export_killCalibrations([1])
  print('\nSASHA: res: %s' % res)
  assert res['OK']
  assert res['Value'][1]['Message'] == 'Permission denied. Calibration with ID 1 has been created by other user.'
  # wrong proxyUserGroup
  calibRun.proxyUserName = 'correctUserName'
  calibRun.proxyUserGroup = 'wrongUserGroup'
  res = calibHandler.export_killCalibration(1, 'dummy')
  assert not res['OK']
  assert res['Message'] == 'Permission denied. Calibration with ID 1 has been created by other user.'
  # everything is correct
  calibRun.proxyUserName = 'correctUserName'
  calibRun.proxyUserGroup = 'correctUserGroup'
  res = calibHandler.export_killCalibration(1, 'dummy')
  assert res['OK']
  assert CalibrationHandler.idsOfCalibsToBeKilled == [1]
  assert calibRun.calibrationFinished
  assert calibRun.resultsSuccessfullyCopied
  assert calibRun.calibrationEndTime is not None

  # clean up
  CalibrationHandler.activeCalibrations = {}


def test_changeDirectoryToCopyTo(calibHandler, mocker):
  """Test changeDirectoryToCopyTo."""
  mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.Operations',
               new=Mock(return_value=Mock(), name='Class'))
  mocker.patch.object(calibHandler, '_getUsernameAndGroup', new=Mock(
      return_value={'OK': True, 'Value': {'username': 'correctUserName', 'group': 'correctUserGroup'}}))
  calibSetting = createCalibrationSettings('CLIC')
  calibSetting.settingsDict['outputPath'] = 'dummy_outputPath'
  calibSetting.settingsDict['outputSE'] = 'dummy_outputSE'
  calibRun = CalibrationRun(33, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  calibRun.proxyUserName = 'correctUserName'
  calibRun.proxyUserGroup = 'correctUserGroup'
  CalibrationHandler.activeCalibrations[33] = calibRun

  # wrong proxyUserName
  res = calibHandler.export_changeDirectoryToCopyTo(33, 'newPath', 'newSE')
  assert res['OK']
  assert CalibrationHandler.activeCalibrations[33].settings['outputPath'] == 'newPath'
  assert CalibrationHandler.activeCalibrations[33].settings['outputSE'] == 'newSE'
  assert not CalibrationHandler.activeCalibrations[33].resultsSuccessfullyCopied

  CalibrationHandler.activeCalibrations[33].resultsSuccessfullyCopied = True
  res = calibHandler.export_changeDirectoryToCopyTo(33, 'newPath2', 'newSE2')
  assert res['OK']
  assert CalibrationHandler.activeCalibrations[33].settings['outputPath'] == 'newPath2'
  assert CalibrationHandler.activeCalibrations[33].settings['outputSE'] == 'newSE2'
  assert not CalibrationHandler.activeCalibrations[33].resultsSuccessfullyCopied

  # clean up
  CalibrationHandler.activeCalibrations = {}


def test_resubmitjobs(calibHandler, mocker):
  """Test rusibmit jobs."""
  tmpMock0 = Mock(return_value={'OK': True, 'Value': []})
  mocker.patch.object(calibHandler, 'export_killCalibration', new=tmpMock0)
  tmpMock1 = Mock(name='calibRunMock')
  tmpMock2 = Mock(return_value=S_OK())
  tmpMock1.submitJobs = tmpMock2
  tmpMock1.settings.__getitem__ = Mock(return_value=5)
  tmpMock1.nFailedJobs = 1
  tmpMock1.calibrationFinished = False

  #  mocker.spy(tmpMock1, 'submitJobs')
  #  mocker.spy(calibHandler, 'export_killCalibration')

  CalibrationHandler.activeCalibrations[27] = tmpMock1
  failedJobs = ((27, 123), (27, 443), (27, 554))

  calibHandler.export_resubmitJobs(failedJobs)  # nJobs == 5; nFailedJobs == 3+1
  #  assert tmpMock1.submitJobs.call_count == 1
  assert tmpMock2.call_count == 1
  assert calibHandler.export_killCalibration.call_count == 0
  calibHandler.export_resubmitJobs(failedJobs)  # nJobs == 5; nFailedJobs == 3+3+1
  #  assert tmpMock1.submitJobs.call_count == 1
  assert tmpMock2.call_count == 1
  assert calibHandler.export_killCalibration.call_count == 1


# TODO this function has decorator... which one need to mock
#  def test_submitJobs(calibHandler, mocker):
#    calibSetting = createCalibrationSettings('CLIC')
#    calibSetting.settingsDict['numberOfJobs'] = 4
#
  #  inputFileDir = {'muon': ['muon1', 'muon2', 'muon3', 'muon4', 'muon5'],
  #                  'kaon': ['kaon1', 'kaon2', 'kaon3', 'kaon4', 'kaon5'],
  #                  'gamma': ['gamma1', 'gamma2', 'gamma3', 'gamma4', 'gamma5'],
  #                  'zuds': ['zuds1', 'zuds2', 'zuds3', 'zuds4', 'zuds5']}
#    res = calibHandler._CalibrationHandler__regroupInputFile(inputFileDir, calibSetting.settingsDict['numberOfJobs'])
#    groupedDict = res['Value']
#
#    mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.saveCalibrationRun', new=Mock(return_value=S_OK()))
#    mocker.patch('ILCDIRAC.CalibrationSystem.Service.CalibrationRun.DiracILC', new=Mock(return_value=S_OK()))
#    testRun  = CalibrationRun(1, groupedDict, calibSetting.settingsDict)
#
#    userJobMock = Mock(name='instance')
#    userJobMock.submit.return_value = 'dummy'
#    userJobMock.append.return_value = {'OK': True}
#
#    res = testRun.submitJobs([])
#    assert res == []
#    assert False

# pylint: disable=protected-access,too-many-public-methods,,no-member


class CalibrationHandlerTest(unittest.TestCase):
  """Test the implementation of the methods of the CalibrationService classes."""

  @classmethod
  def setUpClass(cls):
    """Set up class."""
    CalibrationHandler.initializeHandler(None)

  def setUp(self):
    """Create a CalibrationHandler instance so we can check some basic functionality."""
    self.transport_mock = Mock()
    RequestHandler._rh__initializeClass(Mock(), Mock(), Mock(), Mock())
    self.calh = CalibrationHandler({}, self.transport_mock)
    self.calh.initialize()
    # TODO mock this call:
    #  self.ops.getValue("/AvailableTarBalls/%s/%s/%s/pandoraAnalysisHeadBin" % (self.platform,
    #                                 'pandora_calibration_scripts', self.appversion), None)

  def tearDown(self):
    """Clean up."""
    if os.path.exists('status'):
      try:
        os.remove('status')
      except OSError:
        print("Failed to clean up CalibrationHandler fixture. Cannot delete file 'status'")
        assert False
    # clean up output directory
    for iCalID in list(CalibrationHandler.activeCalibrations.keys()):
      dirToDelete = 'calib%s' % iCalID
      if os.path.exists(dirToDelete):
        try:
          shutil.rmtree(dirToDelete)
        except EnvironmentError as e:
          print("Failed to delete directory: %s" % dirToDelete, str(e))
          assert False
    CalibrationHandler.activeCalibrations = {}
    CalibrationHandler.calibrationCounter = 0

# FIXME: Change tests to reflect new way of starting calibration creation

  #
  #  def test_submitresult_old_stepid( self ):
  #    with patch.object( CalibrationRun, 'submitJobs', new=Mock()):
  #      for _ in xrange( 0, 50 ): #creates Calibrations with IDs 1-50
  #        self.calh.export_createCalibration( '', '', [], 0, '', '' )
  #    CalibrationHandler.activeCalibrations[ 27 ].currentStep = 13
  #    assertDiracSucceeds( self.calh.export_submitResult( 27, 12, 9841, [ 5, 6, 2, 1, 7 ] ), self )
  #    for i in xrange( 0, 30 ):
  #      assertEqualsImproved( CalibrationHandler.activeCalibrations[ 27 ].stepResults[ i ].getNumberOfResults(),
  #                            0, self )
  #
  #  def test_submitresult_wrong_calibrationID( self ):
  #    with patch.object( CalibrationRun, 'submitJobs', new=Mock()):
  #      for _ in xrange( 0, 50 ): #creates Calibrations with IDs 1-50
  #        self.calh.export_createCalibration( '', '', [], 0, '', '' )
  #    res = self.calh.export_submitResult( 54, 1, 9841, [ 5, 6, 2, 1, 7 ] )
  #    assertDiracFailsWith( res, 'Calibration with id 54 not found', self )
  #    for i in xrange( 0, 50 ):
  #      assertEqualsImproved( CalibrationHandler.activeCalibrations[ 27 ].stepResults[ i ].getNumberOfResults(),
  #                            0, self )

  #  def test_createcalibration( self ):
  #    CalibrationHandler.calibrationCounter = 834 - 1 # newly created Calibration gets ID 834
  #    job_mock = Mock()
  #    with patch.object( CalibrationRun, 'submitJobs', new=job_mock ):
  #      result = self.calh.export_createCalibration( 'steeringfile', 'version', [ 'inputfile1', 'inputfile2' ],
  #                                                   12, '', '' )
  #    assertDiracSucceedsWith_equals( result, ( 834, job_mock() ), self )
  #    testRun = CalibrationHandler.activeCalibrations[ 834 ]
  #    assertEqualsImproved(
  #      ( testRun.steeringFile, testRun.softwareVersion, testRun.inputFiles, testRun.numberOfJobs ),
  #      ( 'steeringfile', 'version', [ 'inputfile1', 'inputfile2' ], 12 ), self )
  #    assertEqualsImproved( CalibrationHandler.calibrationCounter, 834, self ) # next calibration gets ID 835
  #
  #  def test_resubmitjobs( self ):
  #    calIDsWorkIDs = [ ( 138, 1249 ), ( 123, 1357 ), ( 498626, 4368 ) ]
  #    CalibrationHandler.activeCalibrations[ 138 ] = CalibrationRun(1, '', '', [], 0 )
  #    CalibrationHandler.activeCalibrations[ 123 ] = CalibrationRun(2, '', '', [], 0 )
  #    CalibrationHandler.activeCalibrations[ 498626 ] = CalibrationRun(3, '', '', [], 0 )
  #    with patch.object( CalibrationRun, 'resubmitJob', new=Mock()) as resubmit_mock:
  #      assertDiracSucceeds( self.calh.export_resubmitJobs( calIDsWorkIDs ), self )
  #      assertEqualsImproved( resubmit_mock.mock_calls, [ call( 1249, proxyUserGroup = '', proxyUserName = '' ),
  #                                                        call( 1357, proxyUserGroup = '', proxyUserName = '' ),
  #                                                        call( 4368, proxyUserGroup = '', proxyUserName = '' ) ],
  #                            self )
  #
  #  def test_resubmitjobs_fails( self ):
  #    calIDsWorkIDs = [ ( 138, 1249 ), ( 198735, 1357 ), ( 498626, 4368 ) ]
  #    CalibrationHandler.activeCalibrations[ 138 ] = CalibrationRun( '', '', [], 0 )
  #    CalibrationHandler.activeCalibrations[ 123 ] = CalibrationRun( '', '', [], 0 )
  #    CalibrationHandler.activeCalibrations[ 498626 ] = CalibrationRun( '', '', [], 0 )
  #    with patch.object( CalibrationRun, 'resubmitJob', new=Mock() ):
  #      res = self.calh.export_resubmitJobs( calIDsWorkIDs )
  #      assertDiracFails( res, self )
  #      assertInImproved( 'Could not resubmit all jobs', res[ 'Message' ], self )
  #      assertInImproved( '[(198735, 1357)]', res[ 'Message' ], self )
  #      assertEqualsImproved( [ ( 198735, 1357 ) ], res[ 'failed_pairs' ], self )

  def test_getnumberofjobs(self):
    """Test get number of jobs."""
    with patch.object(CalibrationRun, 'submitJobs', new=Mock()):
      with patch('ILCDIRAC.CalibrationSystem.Utilities.functions.splitFilesAcrossJobs',
                 new=Mock(return_value={'OK': True, 'Value': []})):
        with patch.object(self.calh, '_getUsernameAndGroup',
                          new=Mock(return_value={'OK': True, 'Value': {'username': 'oviazlo', 'group': 'ilc_users'}})):
          tmpDict = {782145: 815, 72453: 421, 189455: 100, 954692: 0, 29485: 1040}
          for iCalID in tmpDict.keys():  # creates Calibrations with IDs 1-50
            calibSettings = createCalibrationSettings('CLIC')
            calibSettings.settingsDict['outputPath'] = 'dummy_outputPath'
            calibSettings.settingsDict['outputSE'] = 'dummy_outputSE'
            calibSettings.settingsDict['numberOfJobs'] = tmpDict[iCalID]
            iCalibRun = CalibrationRun(iCalID, {'muon': [], 'kaon': [], 'gamma': [],
                                                'zuds': []}, calibSettings.settingsDict)
            CalibrationHandler.activeCalibrations[iCalID] = iCalibRun
          result = self.calh.export_getNumberOfJobsPerCalibration()
          assertDiracSucceeds(result, self)
          assertEqualsImproved(result['Value'], tmpDict, self)

  #  def test_getnewparams_calculationfinished( self ):
  #    # TODO rewrite this test
  #    calibSetting = createCalibrationSettings('CLIC')
  #    testRun  = CalibrationRun(1, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  #    testRun.calibrationFinished = True
  #    CalibrationHandler.activeCalibrations[ 2489 ] = testRun
  #    assertDiracSucceedsWith( self.calh.export_getNewParameters( 2489, 193 ),
  #                             'Calibration finished! End job now', self )

  #  def test_getnewparams_nonewparamsyet( self ):
  #    calibSetting = createCalibrationSettings('CLIC')
  #    testRun  = CalibrationRun(1, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  #    testRun.currentStep = 149
  #    CalibrationHandler.activeCalibrations[ 2489 ] = testRun
  #    assertDiracSucceedsWith_equals(self.calh.export_getNewParameters( 2489, 149 ),
  #                                   None, self)

  def test_getnewparams_newparams(self):
    """Test get new params."""
    calibSetting = createCalibrationSettings('CLIC')
    calibSetting.settingsDict['outputPath'] = 'dummy_outputPath'
    calibSetting.settingsDict['outputSE'] = 'dummy_outputSE'
    testRun = CalibrationRun(1, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
    testRun.currentStep = 36
    testRun.currentParameterSet = {'dummy': 2435}
    testRun.proxyUserName = 'correctUserName'
    testRun.proxyUserGroup = 'correctUserGroup'
    CalibrationHandler.activeCalibrations[2489] = testRun
    with patch.object(CalibrationHandler, '_checkClientRequest', new=Mock(
        return_value=S_OK())):
      res = self.calh.export_getNewParameters(2489, 35)
      print(res)
      assertDiracSucceedsWith_equals(res, testRun.currentParameterSet, self)

  def test_getnewparams_inactive_calibration(self):
    """Test get new params."""
    print (dir(self.calh))
    inputData = {'zuds': ["dummy.slcio"], 'gamma': ["dummy.slcio"], 'muon': ["dummy.slcio"], 'kaon': ["dummy.slcio"]}
    numberOfEventsPerFile = {'zuds': 133, 'gamma': 133, 'muon': 133, 'kaon': 133}
    with patch.object(CalibrationRun, 'submitJobs', new=Mock()):
      with patch('ILCDIRAC.CalibrationSystem.Utilities.functions.splitFilesAcrossJobs',
                 new=Mock(return_value={'OK': True, 'Value': []})):
        with patch.object(self.calh, '_getUsernameAndGroup',
                          new=Mock(return_value={'OK': True, 'Value': {'username': 'oviazlo', 'group': 'ilc_users'}})):
          for _ in range(0, 50):  # creates Calibrations with IDs 1-50
            calibSettings = createCalibrationSettings('CLIC')
            calibSettings.settingsDict['outputPath'] = 'dummy_outputPath'
            calibSettings.settingsDict['outputSE'] = 'dummy_outputSE'
            res = self.calh.export_createCalibration(inputData, numberOfEventsPerFile, calibSettings.settingsDict)
            if not res['OK']:
              print(res['Message'])
              assert False
          res = self.calh.export_getNewParameters(135, 913)
          print('res: %s' % res)
          assertDiracFailsWith(res, 'CalibrationID is not in active calibrations: 135', self)

  #  @patch('DIRAC.ConfigurationSystem.Client.Helpers.Operations.getValue', return_value='dummyReturnString')
  #  def test_endcurrentstep( self, mock_operations ):
  #    from ILCDIRAC.CalibrationSystem.Service.CalibrationHandler import CalibrationResult
  #    with patch.object( CalibrationRun, 'submitJobs', new=Mock()):
  #      self.calh.export_createCalibration('', '', [], 0, '', '' )
  #    self.calh.activeCalibrations[ 1 ].currentStep = 15
  #    result1 = [ 1, 2.3, 5 ]
  #    result2 = [ 0, 0.2, -0.5 ]
  #    result3 = [ -10, -5.4, 2 ]
  #    res = CalibrationResult()
  #    res.addResult( 2384, result1 )
  #    res.addResult( 742, result2 )
  #    res.addResult( 9354, result3 )
  #    self.calh.activeCalibrations[ 1 ].stepResults[ 15 ] = res
  #    self.calh.activeCalibrations[ 1 ].endCurrentStep()
  #    self.assertTrue( self.calh.activeCalibrations[ 1 ].calibrationFinished, 'Expecting calibration to be finished' )
  #
  #  @patch('DIRAC.ConfigurationSystem.Client.Helpers.Operations.getValue', return_value='dummyReturnString')
  #  def test_endcurrentstep_not_finished( self ):
  #    from ILCDIRAC.CalibrationSystem.Service.CalibrationHandler import CalibrationResult
  #    with patch.object( CalibrationRun, 'submitJobs', new=Mock()):
  #      self.calh.export_createCalibration('', '', [], 0, '', '' )
  #    self.calh.activeCalibrations[ 1 ].currentStep = 14
  #    result1 = [ 1, 2.3, 5 ]
  #    result2 = [ 0, 0.2, -0.5 ]
  #    result3 = [ -10, -5.4, 2 ]
  #    res = CalibrationResult()
  #    res.addResult( 2384, result1 )
  #    res.addResult( 742, result2 )
  #    res.addResult( 9354, result3 )
  #    self.calh.activeCalibrations[ 1 ].stepResults[ 14 ] = res
  #    self.calh.activeCalibrations[ 1 ].endCurrentStep()
  #    self.assertFalse( self.calh.activeCalibrations[ 1 ].calibrationFinished,
  #                      'Expecting calibration to be finished' )

  #  def test_addlists_work( self ):
  #    # Simple case
  #    test_list_1 = [1, 148]
  #    test_list_2 = [-3, 0.2]
  #    calibSetting = createCalibrationSettings('CLIC')
  #    calibSetting.settingsDict['outputPath'] = 'dummy_outputPath'
  #    testobj = CalibrationRun(1, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  #    res = testobj._CalibrationRun__addLists(test_list_1, test_list_2)
  #    assertEqualsImproved([-2, 148.2], res, self)
  #
  #  def test_addlists_work_2(self):
  #    # More complex case
  #    test_list_1 = [9013, -137.25, 90134, 4278, -123, 'abc', ['a', False]]
  #    test_list_2 = [0, 93, -213, 134, 98245, 'aifjg', ['some_entry', {}]]
  #    calibSetting = createCalibrationSettings('CLIC')
  #    calibSetting.settingsDict['outputPath'] = 'dummy_outputPath'
  #    testobj = CalibrationRun(1, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  #    res = testobj._CalibrationRun__addLists(test_list_1, test_list_2)
  #    assertEqualsImproved([9013, -44.25, 89921, 4412, 98122, 'abcaifjg',
  #                          ['a', False, 'some_entry', {}]], res, self)
  #
  #  def test_addlists_empty(self):
  #    test_list_1 = []
  #    test_list_2 = []
  #    calibSetting = createCalibrationSettings('CLIC')
  #    calibSetting.settingsDict['outputPath'] = 'dummy_outputPath'
  #    testobj = CalibrationRun(1, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  #    res = testobj._CalibrationRun__addLists(test_list_1, test_list_2)
  #    assertEqualsImproved([], res, self)
  #
  #  def test_addlists_incompatible(self):
  #    test_list_1 = [1, 83, 0.2, -123]
  #    test_list_2 = [1389, False, '']
  #    calibSetting = createCalibrationSettings('CLIC')
  #    calibSetting.settingsDict['outputPath'] = 'dummy_outputPath'
  #    testobj = CalibrationRun(1, {'dummy': ['dummy_inputFiles1', 'dummy_inputFiles2']}, calibSetting.settingsDict)
  #    with pytest.raises(ValueError) as ve:
  #      testobj._CalibrationRun__addLists(test_list_1, test_list_2)
  #    assertInImproved('the two lists do not have the same number of elements', ve.__str__().lower(), self)

  def test_resubmitJob(self):
    """Test resubmit jobs."""
    pass  # FIXME: Finish atest once corresponding method is written



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test CheckCollections module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import CheckCollections
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.CheckCollections'

# pylint: disable=protected-access


class CheckCollectionsTestCase(unittest.TestCase):
  """Base class for the CheckCollations test cases."""

  def setUp(self):
    """set up the objects."""
    self.chcol = CheckCollections({})

  def test_setcollection(self):
    self.assertFalse(self.chcol._errorDict)
    self.chcol.setCollections(['mycollection', True, {'something': 124}])
    self.assertFalse(self.chcol._errorDict)
    assertEqualsImproved(self.chcol.collections, ['mycollection', True, {'something': 124}], self)

  def test_setcollection_invalid(self):
    self.assertFalse(self.chcol._errorDict)
    self.chcol.setCollections({'blabla', True})
    assertEqualsImproved(len(self.chcol._errorDict['_checkArgs']), 1, self)

  def test_applicationModule(self):
    result = self.chcol._applicationModule()
    self.assertIsNotNone(result)
    self.chcol._applicationModuleValues(result)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.chcol._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.chcol._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
        patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.chcol._userjobmodules(None),
                            'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
        patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.chcol._prodjobmodules(None),
                            'prodjobmodules failed', self)

  def test_resolvelinkedparams(self):
    step_mock = Mock()
    input_mock = Mock()
    input_mock.getType.return_value = {'abc': False}
    self.chcol._linkedidx = 3
    self.chcol._jobsteps = [None, None, None, input_mock]
    assertDiracSucceeds(self.chcol._resolveLinkedStepParameters(step_mock), self)
    step_mock.setLink.assert_called_once_with('InputFile', {'abc': False}, 'OutputFile')

  def test_resolvelinkedparams_noinputstep(self):
    self.chcol._linkedidx = None
    self.chcol._inputappstep = []
    assertDiracSucceeds(self.chcol._resolveLinkedStepParameters(None), self)

  def test_checkconsistency(self):
    self.chcol.setCollections(['some_entry', False, 'add_me', {}])
    assertDiracSucceeds(self.chcol._checkConsistency(), self)

  def test_checkconsistency_nocollection(self):
    assertDiracFailsWith(self.chcol._checkConsistency(), 'no collections to check', self)

  def test_checkconsistency_checkfails(self):
    self.chcol.setCollections(['some_entry', False, 'add_me', {}])
    self.chcol._inputapp = ['some_depdency', 'unavailable_dependency_fail_on_this']
    self.chcol._jobapps = ['myjobapp_1', 'some_dependency']
    assertDiracFailsWith(self.chcol._checkConsistency(), 'job order not correct', self)



"""Retrieve MIP Peak calibration constants from Calibration.txt file."""

from __future__ import print_function
from __future__ import absolute_import
import sys
from Helper_Functions import find_between


Calibration_File_And_Path = sys.argv[1]
ECal_Or_HCal = sys.argv[2]

HCalMIPMPV = ''
HCalMIPMPVBarrel = ''
HCalMIPMPVEndCap = ''

with open(Calibration_File_And_Path, 'r') as f:
    searchlines = f.readlines()
    for line in searchlines:
        if 'HCal Barrel MIP Peak' in line:
            HCalMIPMPVBarrel = find_between(line, ' : ', ' :')
        if 'HCal EndCap MIP Peak' in line:
            HCalMIPMPVEndCap = find_between(line, ' : ', ' :')
        if 'ECal MIP Peak' in line:
            ECalMIPMPV = find_between(line, ' : ', ' :')

# HCalMIPMPV = str( float(HCalMIPMPVBarrel) + float (HCalMIPMPVEndCap) / 2)

if 'ECal' in ECal_Or_HCal:
    print (str(ECalMIPMPV))
elif 'HCal' in ECal_Or_HCal:
    print (str(HCalMIPMPVBarrel))



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Command Line Parameters for Replication transformation scripts."""

from __future__ import absolute_import
import os

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
from DIRAC.TransformationSystem.Utilities.ReplicationCLIParameters import Params as DParams

VALIDDATATYPES = ('GEN', 'SIM', 'REC', 'DST')

LOG = gLogger.getSubLogger(__name__)


def checkDatatype(prodID, datatype):
  """Check if the datatype makes sense for given production."""
  # skip data type check when creating replications in development for prod productions this check doesn't work
  if os.environ.get('SKIP_CHECK', False):
    LOG.warn("Skipping Datatype check!")
    return S_OK()

  tClient = TransformationClient()
  cond = dict(TransformationID=prodID)
  trafo = tClient.getTransformations(cond)
  if not trafo['OK']:
    return trafo
  if len(trafo['Value']) != 1:
    return S_ERROR("Did not get unique production for this prodID")

  trafoType = trafo['Value'][0]['Type'].split("_")[0]

  dataTypes = Operations().getOptionsDict('Production/TransformationDatatypes')
  if not dataTypes['OK']:
    return dataTypes

  dataTypes = dataTypes['Value']
  if trafoType not in dataTypes[datatype]:
    return S_ERROR("Datatype %r doesn't fit production type %r for prodID %s" % (datatype, trafoType, prodID))

  return S_OK()


def getTransformationGroup(prodID, groupName):
  """Return TransformationGroup of prodID."""
  if groupName:
    return groupName
  tClient = TransformationClient()
  res = tClient.getTransformationParameters(prodID, 'TransformationGroup')
  if not res['OK']:
    return None
  return res['Value']


class Params(DParams):
  """Command line parameter class."""

  def __init__(self):
    super(Params, self).__init__()
    self.datatype = None
    self.forcemoving = False
    self.allFor = []

  def setAllFor(self, allFor):
    self.allFor = allFor
    return S_OK()

  def setDatatype(self, datatype):
    if datatype.upper() not in VALIDDATATYPES:
      self.errorMessages.append("ERROR: Unknown Datatype, use %s " % (",".join(VALIDDATATYPES),))
      return S_ERROR()
    self.datatype = datatype.upper()
    return S_OK()

  def setForcemoving(self, _forcemoving):
    self.forcemoving = True
    return S_OK()

  def registerSwitches(self, script):
    """register command line arguments.

    :param script: Dirac.Core.Base Script Class
    """
    super(Params, self).registerSwitches(script)

    useMessage = []
    useMessage.append('%s <prodID> <TargetSEs> <SourceSEs> {GEN,SIM,REC,DST} -NExtraName [-F] [-G 10]'
                      % script.scriptName)
    useMessage.extend(['', 'or', ''])
    useMessage.append('%s --AllFor="<prodID1>, <prodID2>, ..." <TargetSEs> <SourceSEs> -NExtraName [-F] [-G 10]'
                      % script.scriptName)
    script.setUsageMessage('\n'.join(useMessage))

  def checkSettings(self, script, checkArguments=False):
    """Check if all required parameters are set, print error message and return S_ERROR if not."""
    super(Params, self).checkSettings(script, checkArguments=checkArguments)

    args = script.getPositionalArgs()
    if len(args) == 4:
      self.__setFourParameters(args)
    elif len(args) == 2 and self.allFor:
      self.__setAllForParameters(args)
    else:
      self.errorMessages.append("ERROR: Not enough arguments")

    if not self.errorMessages:
      return S_OK()
    LOG.error("\n".join(self.errorMessages))
    script.showHelp()
    return S_ERROR()

  def __setFourParameters(self, args):
    """Set parameters if four command line arguments are given."""
    self.setMetaValues(args[0])
    # cast the metaValues to int
    self.metaValues = [int(val) for val in self.metaValues]
    self.setTargetSE(args[1])
    self.setSourceSE(args[2])
    self.setDatatype(args[3])

  def __setAllForParameters(self, args):
    """Set parameters if two command line arguments are given along with allFor."""
    self.setMetaValues(self.allFor)
    self.metaValues = [int(val) for val in self.metaValues]
    # place the indiviual entries as well.
    prodTemp = list(self.metaValues)
    for prodID in prodTemp:
      self.metaValues.append(prodID + 1)
      self.metaValues.append(prodID + 2)
    self.metaValues = sorted(self.metaValues)
    self.setTargetSE(args[0])
    self.setSourceSE(args[1])



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""utilities to treat Marlin XML Steering files.

Implementations for Overlay processors can handle OverlayTiming and Overlay processor Type
"""

from __future__ import absolute_import
from xml.etree.ElementTree import Comment

from DIRAC import S_OK, S_ERROR

from ILCDIRAC.Core.Utilities.OverlayFiles import getOverlayFiles

DEFAULT_OVERLAY_PROCESSORS = ['overlaytiming', 'bgoverlay']


def setOverlayFilesParameter(tree, overlayParam=None):
  """set the parameters for overlay processors in MarlinSteering xml.

  treat processors and groups of processors

  :param tree: XML tree of marlin steering file
  :param overlayParam: list of three tuples of backgroundType, eventsPerBackgroundFile, processorName
  """

  overlayActive = __checkOverlayActive(tree)
  if not overlayParam and not overlayActive:
    return S_OK()

  if not overlayParam and overlayActive:
    return S_ERROR("Found active overlay processors, but no overlayInput was run")

  for backgroundType, eventsPerBackgroundFile, processorName in overlayParam:
    processorsToCheck = [processorName] if processorName else DEFAULT_OVERLAY_PROCESSORS
    for processorType in processorsToCheck:
      resOT = __checkOverlayProcessor(tree, eventsPerBackgroundFile, processorType.lower(), backgroundType)
      if not resOT['OK']:
        return resOT
      resGroupO = __checkOverlayGroup(tree, eventsPerBackgroundFile, processorType.lower(), backgroundType)
      if not resGroupO['OK']:
        return resGroupO

  return S_OK()


def __checkOverlayActive(tree):
  """checks if the overlayProcessor would actually overlay anything, or if the parameters for number of events are 0.

  :returns: True or False
  :rtype: bool
  """
  overlay = False
  processors = tree.findall('execute/processor')
  for processor in processors:
    processorName = processor.attrib.get('name', '').lower()
    if any(processorName.count(pattern.lower()) for pattern in DEFAULT_OVERLAY_PROCESSORS):
      overlay = True
  if not overlay:
    return False

  for processor in tree.findall('processor'):
    processorName = processor.attrib.get('name', '').lower()
    processorType = processor.attrib.get('type', '').lower()
    if any(processorName.count(pattern.lower()) for pattern in DEFAULT_OVERLAY_PROCESSORS) or \
       any(processorType.count(pattern.lower()) for pattern in DEFAULT_OVERLAY_PROCESSORS):
      for param in processor.findall('parameter'):
        if param.attrib.get('name') in ('NumberBackground', 'NBunchtrain', 'expBG') and \
           (param.attrib.get('value') in ('0', '0.0') or param.text in ('0', '0.0')):
          return False

  return True


def __checkOverlayGroup(tree, eventsPerBackgroundFile, processorType, bkgType):
  """check if there is an OverlayProcessor, also handling overlay processors that get parameters from a group."""
  groups = tree.findall('group')
  for group in groups:
    groupParameters = group.findall('parameter')
    resG = __checkOverlayProcessor(group, eventsPerBackgroundFile, processorType, bkgType, groupParameters)
    if not resG['OK']:
      return resG
  return S_OK()


def __checkOverlayProcessor(tree, eventsPerBackgroundFile, processorType, bkgType, groupParameters=None,):
  """check the for the overlay processor *processorType* and set the appropriate parameter values."""

  for processor in tree.findall('processor'):
    if processor.attrib.get('name', '').lower().count(processorType.lower()) or \
       processor.attrib.get('type', '').lower().count(processorType.lower()):
      files = getOverlayFiles(bkgType)
      if not files:
        return S_ERROR('Could not find any overlay files')
      if 'overlaytiming' in processor.attrib.get('type', '').lower():
        __changeProcessorTagValue(
            processor,
            'parameter',
            'BackgroundFileNames',
            '\n'.join(files),
            "Overlay files changed",
            groupParameters)
      if processor.attrib.get('type', '').lower() == 'overlay':
        __changeProcessorTagValue(processor, 'parameter', "InputFileNames", "\n".join(files), "Overlay files changed")
        __changeProcessorTagValue(processor, 'parameter', "NSkipEventsRandom",
                                   "%d" % int(len(files) * eventsPerBackgroundFile), "NSkipEventsRandom Changed")

  return S_OK()


def setOutputFileParameter(tree, outputFile, outputREC, outputDST):
  for processor in tree.findall('processor'):
    if 'name' not in processor.attrib:
      continue
    if outputFile:
      if processor.attrib.get('name') == 'MyLCIOOutputProcessor' \
         or processor.attrib.get('type') == 'LCIOOutputProcessor':
        __changeProcessorTagValue(processor, 'parameter', 'LCIOOutputFile', outputFile, 'output file changed')
    else:
      if outputREC:
        if processor.attrib.get('name') in ('MyLCIOOutputProcessor', 'Output_REC'):
          __changeProcessorTagValue(processor, 'parameter', 'LCIOOutputFile', outputREC, 'REC file changed')
      if outputDST:
        if processor.attrib.get('name') in ('DSTOutput', 'Output_DST'):
          __changeProcessorTagValue(processor, 'parameter', 'LCIOOutputFile', outputDST, 'DST file changed')
  return S_OK()


def __changeProcessorTagValue(processor, tagTypename, parameterName, newValue, newComment, additionalParameters=None):
  """modify the value of the tag with *tagTypename* with name attribute equall to *parameterName* to *newValue*

  :param processor: XML object representing processor
  :param tagTypename: name of the tag to look for, e.g. <parameter /> to find all parameters for given processor section
  :param parameterName: name of the parameter to change <parameter name="parameterName"/>
  :param newValue: new value to set for the parameter
  :param newComment: comment to set
  """
  tags = processor.findall(tagTypename) + (additionalParameters if additionalParameters else [])
  for tag in tags:
    if tag.attrib.get('name') == parameterName:
      tag.text = newValue
      com = Comment(newComment)
      processor.insert(0, com)
